"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TradeOrderClient = void 0;
const common_sdk_1 = require("@orca-so/common-sdk");
const __1 = require("../");
const types_1 = require("../types");
class TradeOrderClient {
    constructor(ctx, publicKey, data, pda) {
        this.ctx = ctx;
        this.data = data;
        this.publicKey = publicKey;
        this.pda = pda;
    }
    static getOrder(ctx, configId, orderId) {
        return __awaiter(this, void 0, void 0, function* () {
            const pda = new __1.PDA(ctx.program.programId, configId);
            const tradeOrderPublickey = pda.getTradeOrderPDA(orderId).key;
            const tradeOrderAccount = (yield ctx.program.account.tradeOrder.fetch(tradeOrderPublickey));
            if (!tradeOrderAccount) {
                throw new Error(`Trade order ${orderId} not found`);
            }
            return new TradeOrderClient(ctx, tradeOrderPublickey, tradeOrderAccount, pda);
        });
    }
    static makeSellOrder(ctx, configId, seller, tokenMint, tokenAmount, fiatAmount) {
        return __awaiter(this, void 0, void 0, function* () {
            const pda = new __1.PDA(ctx.program.programId, configId);
            const configAddress = pda.getConfigPDA().key;
            let configAccount = (yield ctx.program.account.escrowConfig.fetch(configAddress));
            if (!configAccount) {
                throw new Error(`Escrow config ${configAddress} not found`);
            }
            const tradeOrderPDA = pda.getTradeOrderPDA(configAccount.currentId);
            const tokenVaultPDA = pda.getTokenVaultPDA(configAccount.currentId);
            const sellerTokenAccount = yield (0, common_sdk_1.deriveATA)(seller, tokenMint);
            const tx = (yield ctx.ixs.makeOrderIx({
                escrowConfig: configAddress,
                side: types_1.OrderSide.Sell,
                tradeOrderPDA: tradeOrderPDA,
                tokenMint: tokenMint,
                userTokenAccount: sellerTokenAccount,
                tokenVault: tokenVaultPDA,
                user: seller,
                tokenAmount: tokenAmount,
                fiatAmount: fiatAmount,
            })).toTx();
            return { tx, id: configAccount.currentId };
        });
    }
    static makeBuyOrder(ctx, configId, buyer, tokenMint, tokenAmount, fiatAmount) {
        return __awaiter(this, void 0, void 0, function* () {
            const pda = new __1.PDA(ctx.program.programId, configId);
            const configAddress = pda.getConfigPDA().key;
            let configAccount = (yield ctx.program.account.escrowConfig.fetch(configAddress));
            if (!configAccount) {
                throw new Error(`Escrow config ${configAddress} not found`);
            }
            const tradeOrderPDA = pda.getTradeOrderPDA(configAccount.currentId);
            const tokenVaultPDA = pda.getTokenVaultPDA(configAccount.currentId);
            const buyerTokenAccount = yield (0, common_sdk_1.deriveATA)(buyer, tokenMint);
            const tx = yield (yield ctx.ixs.makeOrderIx({
                escrowConfig: configAddress,
                side: types_1.OrderSide.Buy,
                tradeOrderPDA: tradeOrderPDA,
                tokenMint: tokenMint,
                userTokenAccount: buyerTokenAccount,
                tokenVault: tokenVaultPDA,
                user: buyer,
                tokenAmount: tokenAmount,
                fiatAmount: fiatAmount,
            })).toTx();
            return { tx, id: configAccount.currentId };
        });
    }
    cancelOrder(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const { tokenMint, tokenVault } = this.data;
            // Get or create canceller token account
            const _a = yield (0, common_sdk_1.resolveOrCreateATA)(this.ctx.connection, user, tokenMint, () => this.ctx.fetcher.getTokenAccountRentExempt(), undefined, user), { address: userTokenAccount } = _a, instruction = __rest(_a, ["address"]);
            const tx = (yield this.ctx.ixs.cancelOrderIx({
                tradeOrder: this.publicKey,
                user: user,
                userTokenAccount: userTokenAccount,
                tokenVault,
            }))
                .toTx()
                .prependInstruction(instruction);
            return tx;
        });
    }
    joinOrder(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const { tokenMint, tokenVault } = this.data;
            let prependIxs = [];
            // get config client
            const configClient = yield __1.EscrowConfigClient.getConfig(this.ctx, this.data.escrowConfig);
            // Get or create joinner token account
            const _a = yield (0, common_sdk_1.resolveOrCreateATA)(this.ctx.connection, user, tokenMint, () => this.ctx.fetcher.getTokenAccountRentExempt(), undefined, user), { address: userTokenAccount } = _a, instruction = __rest(_a, ["address"]);
            prependIxs.push(instruction);
            // Get or create default joinner kyc cert
            const userKycCertPda = configClient.pda.getKycCertPDA(user);
            const accountInfo = yield this.ctx.connection.getAccountInfo(userKycCertPda.key);
            if (accountInfo == null) {
                const initKycCertIx = (yield this.ctx.ixs.initKycCertIx({
                    escrowConfig: configClient.publicKey,
                    caller: user,
                    user,
                    kycCertPubkey: userKycCertPda.key,
                    kycCertBump: userKycCertPda.bump,
                })).ix;
                if (!initKycCertIx) {
                    throw new Error("Fail to get the initialize kyc cert instruction");
                }
                prependIxs.push(initKycCertIx);
            }
            const tx = (yield this.ctx.ixs.joinSellOrderIx({
                escrowConfig: configClient.publicKey,
                tradeOrder: this.publicKey,
                user,
                userKycCert: userKycCertPda.key,
                userTokenAccount,
                tokenVault,
            }))
                .toTx()
                .prependInstructions(prependIxs);
            return tx;
        });
    }
    release() {
        return __awaiter(this, void 0, void 0, function* () {
            const { sellerAuthority, buyerAuthority, tokenVault } = this.data;
            const _a = yield (0, common_sdk_1.resolveOrCreateATA)(this.ctx.connection, sellerAuthority, this.data.tokenMint, () => this.ctx.fetcher.getTokenAccountRentExempt(), undefined, sellerAuthority), { address: sellerTokenAccount } = _a, createSellerTokenAccountInstruction = __rest(_a, ["address"]);
            const _b = yield (0, common_sdk_1.resolveOrCreateATA)(this.ctx.connection, buyerAuthority, this.data.tokenMint, () => this.ctx.fetcher.getTokenAccountRentExempt(), undefined, sellerAuthority), { address: buyerTokenAccount } = _b, createBuyerTokenAccountInstruction = __rest(_b, ["address"]);
            const tx = (yield this.ctx.ixs.releaseIx({
                tradeOrder: this.publicKey,
                seller: sellerAuthority,
                sellerTokenAccount,
                buyerTokenAccount,
                tokenVault,
            }))
                .toTx()
                .prependInstructions([
                createSellerTokenAccountInstruction,
                createBuyerTokenAccountInstruction,
            ]);
            return tx;
        });
    }
    callArbitrator(caller) {
        return __awaiter(this, void 0, void 0, function* () {
            const tx = (yield this.ctx.ixs.callArbitrator({
                escrowConfig: this.data.escrowConfig,
                tradeOrder: this.publicKey,
                caller: caller,
            })).toTx();
            return tx;
        });
    }
    resolveDispute(arbitrator, disputeAction, compensationAction) {
        return __awaiter(this, void 0, void 0, function* () {
            // Get or create corressponding token accounts of parties if needed
            const _a = yield (0, common_sdk_1.resolveOrCreateATA)(this.ctx.connection, arbitrator, this.data.tokenMint, () => this.ctx.fetcher.getTokenAccountRentExempt(), undefined, arbitrator), { address: arbitratorTokenAccount } = _a, createArbitratorTokenAccountInstruction = __rest(_a, ["address"]);
            const _b = yield (0, common_sdk_1.resolveOrCreateATA)(this.ctx.connection, this.data.sellerAuthority, this.data.tokenMint, () => this.ctx.fetcher.getTokenAccountRentExempt(), undefined, arbitrator), { address: sellerTokenAccount } = _b, createSellerTokenAccountInstruction = __rest(_b, ["address"]);
            const _c = yield (0, common_sdk_1.resolveOrCreateATA)(this.ctx.connection, this.data.buyerAuthority, this.data.tokenMint, () => this.ctx.fetcher.getTokenAccountRentExempt(), undefined, arbitrator), { address: buyerTokenAccount } = _c, createBuyerTokenAccountInstruction = __rest(_c, ["address"]);
            const tx = (yield this.ctx.ixs.resolveDispute({
                disputeAction,
                compensationAction,
                escrowConfig: this.data.escrowConfig,
                tradeOrder: this.publicKey,
                arbitratorAuthority: arbitrator,
                arbitartorCert: this.pda.getArbitratorCertPDA(arbitrator).key,
                arbitratorTokenAccount: arbitratorTokenAccount,
                seller: this.data.sellerAuthority,
                sellerTokenAccount: sellerTokenAccount,
                buyerTokenAccount: buyerTokenAccount,
                tokenVault: this.data.tokenVault,
            }))
                .toTx()
                .prependInstructions([
                createArbitratorTokenAccountInstruction,
                createBuyerTokenAccountInstruction,
                createSellerTokenAccountInstruction,
            ]);
            return tx;
        });
    }
    refresh() {
        return __awaiter(this, void 0, void 0, function* () {
            const tradeOrderAccount = (yield this.ctx.program.account.tradeOrder.fetch(this.publicKey));
            this.data = tradeOrderAccount;
            return this;
        });
    }
}
exports.TradeOrderClient = TradeOrderClient;
